#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_form.interfaces import get_form_weight
from pyams_form.interfaces.form import IFormWidgetsGroup, IGroupsBasedForm, IInnerSubForm

# import packages
from pyramid.decorator import reify
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


@implementer(IFormWidgetsGroup)
class FormWidgetsGroup(object):
    """Form widgets group"""

    form = None
    id = FieldProperty(IFormWidgetsGroup['id'])
    widgets = FieldProperty(IFormWidgetsGroup['widgets'])
    bordered = FieldProperty(IFormWidgetsGroup['bordered'])
    legend = FieldProperty(IFormWidgetsGroup['legend'])
    help = FieldProperty(IFormWidgetsGroup['help'])
    _css_class = FieldProperty(IFormWidgetsGroup['css_class'])
    switch = FieldProperty(IFormWidgetsGroup['switch'])
    checkbox_switch = FieldProperty(IFormWidgetsGroup['checkbox_switch'])
    checkbox_field = FieldProperty(IFormWidgetsGroup['checkbox_field'])
    checkbox_widget = FieldProperty(IFormWidgetsGroup['checkbox_widget'])
    checkbox_mode = FieldProperty(IFormWidgetsGroup['checkbox_mode'])
    hide_if_empty = FieldProperty(IFormWidgetsGroup['hide_if_empty'])
    subforms_legend = FieldProperty(IFormWidgetsGroup['subforms_legend'])

    def __init__(self, form, id, widgets=None, bordered=True, legend=None, help=None, css_class='', switch=False,
                 checkbox_switch=False, checkbox_field=None, checkbox_mode='hide', hide_if_empty=False):
        assert (not checkbox_switch) or checkbox_field, "You must define checkbox field when using checkbox switch"
        self.form = form
        self.id = id
        self.widgets = widgets or []
        self.bordered = bordered
        self.legend = legend
        self.help = help
        self._css_class = css_class
        self.switch = switch
        self.checkbox_switch = checkbox_switch
        self.checkbox_field = checkbox_field
        self.checkbox_mode = checkbox_mode
        self.hide_if_empty = hide_if_empty

    @property
    def css_class(self):
        css_class = self._css_class
        if self.switch:
            if self.checkbox_switch:
                css_class += ' checker'
            else:
                css_class += ' switcher'
        return css_class

    @property
    def checkbox_widget(self):
        if self.checkbox_field is None:
            return None
        for widget in self.widgets:
            if widget.field is self.checkbox_field:
                return widget

    @reify
    def visible(self):
        if self.checkbox_switch:
            widget = self.checkbox_widget
            context = widget.context
            name = widget.field.getName()
            value = getattr(widget.field.interface(context), name, None)
            return bool(value)
        else:
            if not (self.switch and self.hide_if_empty):
                return True
            for widget in self.widgets:
                if not widget.ignoreContext:
                    field = widget.field
                    context = widget.context
                    name = field.getName()
                    value = getattr(field.interface(context), name, None)
                    if value and (value != field.default):
                        return True
            return False

    @property
    def visible_widgets(self):
        for widget in self.widgets:
            if (self.checkbox_field is None) or (widget.field is not self.checkbox_field):
                yield widget

    @property
    def switchable(self):
        return self.switch or self.checkbox_switch

    @property
    def switcher_state(self):
        return 'on' if self.visible else 'off'

    @property
    def checker_state(self):
        return 'on' if self.visible else 'off'

    @reify
    def subforms(self):
        registry = self.form.request.registry
        return sorted((adapter[1]
                       for adapter in registry.getAdapters((self.form.context, self.form.request, self),
                                                           IInnerSubForm)),
                      key=get_form_weight)


def NamedWidgetsGroup(form, id, widgets, names=(), bordered=True, legend=None, help=None, css_class='', switch=False,
                      checkbox_switch=False, checkbox_field=None, checkbox_mode='hide', hide_if_empty=False):
    """Create a widgets group based on widgets names"""
    return FormWidgetsGroup(form, id, [widgets.get(name) for name in names], bordered, legend, help, css_class, switch,
                            checkbox_switch, checkbox_field, checkbox_mode, hide_if_empty)


@implementer(IGroupsBasedForm)
class GroupsBasedForm(object):
    """Groups based form

    Should be used as a base class for forms also implementing IForm
    """

    main_group_legend = FieldProperty(IGroupsBasedForm['main_group_legend'])
    main_group_class = FieldProperty(IGroupsBasedForm['main_group_class'])

    def __init__(self):
        self._groups = []

    def add_group(self, group):
        self._groups.append(group)

    @property
    def groups(self):
        result = self._groups[:]
        others = []
        for widget in self.widgets.values():
            found = False
            for group in result:
                if widget in group.widgets:
                    found = True
                    break
            if not found:
                others.append(widget)
        if others:
            result.insert(0, FormWidgetsGroup(self, None, others,
                                              legend=self.main_group_legend,
                                              css_class=self.main_group_class))
        return result
